@@ -1,6 +1,6 @@
 """
 
-Copyright 2019 European Union
+Copyright 2019-2020 European Union
 
 Licensed under the EUPL, Version 1.2 or as soon they will be approved by the European Commission  subsequent versions of the EUPL (the "Licence");
 
@@ -23,10 +23,12 @@ from pathlib import Path
 import xarray as xr
 import numpy as np
 
-from lisfloodutilities.readers import PCRasterMap
+from .helpers import pcraster_command
+from ..readers.pcr import PCRasterMap
+from .. import version
 
 logging.basicConfig(level=logging.INFO)
-logger = logging.getLogger()
+logger = logging.getLogger('CUTMAPS')
 
 
 def cutmap(f, fileout, x_min, x_max, y_min, y_max):
@@ -35,40 +37,38 @@ def cutmap(f, fileout, x_min, x_max, y_min, y_max):
     logger.info('Variable: %s', var)
 
     if isinstance(x_min, float):
-        # bounding box input from user
-        sliced_var = _cut_from_coords(nc, var, x_min, x_max, y_min, y_max)
+        # bounding box input from user  # FIXME weak isistance test
+        sliced_var = cut_from_coords(nc, var, x_min, x_max, y_min, y_max)
     else:
         # user provides with indices directly (not coordinates)
-        sliced_var = _cut_from_indices(nc, var, x_min, x_max, y_min, y_max)
+        sliced_var = cut_from_indices(nc, var, x_min, x_max, y_min, y_max)
 
     if sliced_var is not None:
-        logger.info('Creating: %s', fileout)
         if 'missing_value' in sliced_var.encoding:
             sliced_var.encoding['_FillValue'] = sliced_var.encoding['missing_value']
+        logger.info('Creating: %s', fileout)
         sliced_var.to_netcdf(fileout)
     if 'laea' in nc.variables or 'lambert_azimuthal_equal_area' in nc.variables:
         var = nc.variables['laea'] if 'laea' in nc.variables else nc.variables['lambert_azimuthal_equal_area']
+        logger.info('Found projection variable: %s', var)
         xr.DataArray(name='laea', data=var.data, dims=var.dims, attrs=var.attrs).to_netcdf(fileout, mode='a')
-    # TODO add global attrs
-    """
-    // global attributes:
-    :history = "Created Tue Feb 03 18:06:52 2015";
-    :conventions = "CF-1.6";
-    :source_software = "Python netCDF3_Classic";
-    :title = "Lisflood maps for European setting Januar 2015";
-    :keywords = "Lisflood, Europe";
-    :source = "Lisflood European maps - pb2015";
-    :institition = "JRC H01";
-    """
+
+    # adding global attributes
     nc_out, _ = open_dataset(fileout)
     nc_out.attrs = nc.attrs
     nc_out.attrs['conventions'] = 'CF-1.6'
     nc_out.attrs['institution'] = 'JRC E1'
-    nc_out.attrs['Source_Software'] = 'lisfloodutilities cutmaps 0.12.12'
-    nc_out.attrs['source_software'] = 'lisfloodutilities cutmaps 0.12.12'
+    nc_out.attrs['Conventions'] = 'CF-1.6'
+    nc_out.attrs['Institution'] = 'JRC E1'
+    nc_out.attrs['Source_Software'] = 'lisfloodutilities cutmaps {}'.format(version)
+    nc_out.attrs['source_software'] = 'lisfloodutilities cutmaps {}'.format(version)
     nc_out.close()
-    nc_out.to_netcdf(fileout, 'a')
-    nc.close()
+    try:
+        nc_out.to_netcdf(fileout, 'a')
+    except ValueError:
+        logger.warning('Cannot add global attributes to %s', fileout)
+    finally:
+        nc.close()
 
 
 def open_dataset(f):
@@ -77,11 +77,11 @@ def open_dataset(f):
         num_dims = 3
     except Exception:  # file has no time component
         num_dims = 2
-        nc = xr.open_dataset(f)
+        nc = xr.open_dataset(f, decode_cf=False)
     return nc, num_dims
 
 
-def _cut_from_indices(nc, var, x_min, x_max, y_min, y_max):
+def cut_from_indices(nc, var, x_min, x_max, y_min, y_max):
     # note: netcdf has lats on first dimension e.g. y_min:y_max are Y/lat dimension indices
     # that in nc file are stored on first dimension: ta(time, lat, lon)
     # you can always adjust indices in input in order to match your nc files structure
@@ -92,9 +92,9 @@ def _cut_from_indices(nc, var, x_min, x_max, y_min, y_max):
     return sliced_var
 
 
-def _cut_from_coords(nc, var, x_min, x_max, y_min, y_max):
+def cut_from_coords(nc, var, x_min, x_max, y_min, y_max):
     # we have coordinates bounds and not indices yet
-    if 'lats' in nc.variables:
+    if 'lat' in nc.variables:
         lats = nc.variables['lat'][:]
         lons = nc.variables['lon'][:]
     else:
@@ -117,15 +117,12 @@ def _cut_from_coords(nc, var, x_min, x_max, y_min, y_max):
     return sliced_var
 
 
-def get_filelist(filelist=None, input_folder=None, glofas_folder=None):
+def get_filelist(input_folder=None, static_data_folder=None):
     list_to_cut = []
-    if filelist:
-        list_to_cut = open(filelist).readlines()
-        list_to_cut = [Path(l.strip()) for l in list_to_cut]
-    elif input_folder:
+    if input_folder:
         list_to_cut = [f for f in Path(input_folder).glob('**/*.nc')]
-    elif glofas_folder:
-        list_to_cut = [f for f in Path(glofas_folder).glob('**/*') if '/.git/' not in f.as_posix()]
+    elif static_data_folder:
+        list_to_cut = [f for f in Path(static_data_folder).glob('**/*') if '/.git/' not in f.as_posix()]
     logger.info('==================> Going to cut %d files', len(list_to_cut))
     return list_to_cut
 
@@ -162,3 +159,60 @@ def get_cuts(cuts=None, mask=None):
         sys.exit(1)
     logger.info('CUTS: \nmin x: %s \nmax x: %s \nmin y: %s \nmax y: %s', x_min, x_max, y_min, y_max)
     return x_min, x_max, y_min, y_max
+
+
+def mask_from_ldd(ldd_map, stations):
+    """
+
+    """
+    try:
+        from pcraster import accuflux
+    except ImportError as e:
+        logger.error('PCRaster not installed. Try to install PCRaster >= 4.3.0 in a conda environment '
+                     'and then install lisfloodutilities with pip')
+        raise e
+
+    path = os.path.dirname(stations)
+
+    station_map = os.path.join(path, 'outlet.map')
+    pcraster_command(cmd='col2map F0 F1 -N --clone F2 --large', files=dict(F0=stations, F1=station_map, F2=ldd_map))
+
+    # accuflux_map = os.path.join(path, 'accuflux.map')
+    # pcraster_command(cmd="pcrcalc 'F0 = accuflux(F1,1)'", files=dict(F0=accuflux_map, F1=ldd_map))
+
+    tmp_txt = os.path.join(path, 'tmp.txt')
+    tmp_map = os.path.join(path, 'tmp.map')
+    with open(stations) as f:
+        for line in f.readlines():
+            x, y, idx = line.split()
+            with open(tmp_txt, "w") as f1:
+                f1.write("%s %s %s\n" % (x, y, 1))
+            catchment_map = os.path.join(path, 'catchmask%05d.map' % int(idx))
+            pcraster_command(cmd='col2map F0 F1 -N --clone F2 --large', files=dict(F0=tmp_txt, F1=tmp_map, F2=ldd_map))
+            pcraster_command(cmd="pcrcalc 'F0 = boolean(catchment(F1, F2))'", files=dict(F0=catchment_map, F1=ldd_map, F2=tmp_map))
+            pcraster_command(cmd="pcrcalc 'F0 = if((scalar(F0) gt (scalar(F0) * 0)) then F0)'", files=dict(F0=catchment_map))
+    os.unlink(tmp_txt)
+    os.unlink(tmp_map)
+
+    regions_map = os.path.join(path, 'area_mask_regions.map')
+    smallmask_map = os.path.join(path, 'mask.map')
+    tempmask_map = os.path.join(path, 'tempmask.map')
+    # init area map
+    pcraster_command(cmd="pcrcalc 'F0 = scalar(F1) * 0 - 1'", files=dict(F0=regions_map, F1=ldd_map))
+    with open(stations) as f:
+        for line in f.readlines():
+            x, y, idx = line.split()
+            catchment_map = os.path.join(path, "catchmask%05d.map" % int(idx))
+
+            pcraster_command(cmd="pcrcalc 'F0 = F0 * (1-scalar(cover(F1,0)))'",
+                             files=dict(F0=regions_map, F1=catchment_map))
+            pcraster_command(cmd="pcrcalc 'F0 = F0 + scalar(cover(F1, 0)) * %d'" % int(idx),
+                             files=dict(F0=regions_map, F1=catchment_map))
+            os.unlink(catchment_map)
+    pcraster_command(cmd="pcrcalc 'F0 = boolean(if(scalar(F1) != -1, scalar(1)))'",
+                     files=dict(F0=tempmask_map, F1=regions_map))
+    pcraster_command(cmd='resample -c 0 F0 F1', files=dict(F0=tempmask_map, F1=smallmask_map))
+    # os.unlink(accuflux_map)
+    os.unlink(tempmask_map)
+    os.unlink(regions_map)
+    return smallmask_map
